﻿#include "VPOUT.h"

//Память для цепочки DMA
ChainDmaPort chain[4];

void VPOUT_Init(unsigned int _div_VCLKO_out, //Fc   Freq_VCLKO_out = 240*1e6/_div_VCLKO_out
                unsigned int _hstart,        //Hstart - количество тактов VCLKO_out(пиксельной синхронизации), при которых HSYNC_out = 0
                unsigned int _hend,          //Период сигнала HSYNC_out в размерности пиксельной синхронизации; Th = HEND*Tc (где Tc - период VCLKO_out)
                unsigned int _vstart,        //VSTART - количество тактов HSYNC_out(строчной синхронизации), при которых VSYNC_out = 0
                unsigned int _vend,          //Период сигнала VSYNC_out в размерности строчной синхронизации; Tv = VEND*Th (где Th - период HSYNC_out)
                unsigned int _frame_addr,    //Адрес памяти, который хранит видеокадр
                unsigned int _frame_len      //Длина кадра в пикселях

)
{

    // Очистка(CLR)
    CSR_VPOUT = (1<<31);

    // EN_VCLKO, EN_HSYNC, EN_VSYNC, Установка режима, в котором синхроимпульс генерирует сам порт
    CSR_VPOUT = (1<<22) | (1<<21) | (1<<20);

    //Разрешаем прерывание VPOUT по концу кадра
    CSR_VPOUT |= (1<<26);

    HstartHend_VPOUT = ((_hstart & 0x3FFF) << 16) | (_hend & 0xFFF);
    VstartVend_VPOUT = ((_vstart & 0x3FFF) << 16) | (_vend & 0xFFF);

    DIV_VPOUT = _div_VCLKO_out;
    
    //Размер кадра в байтах, так как каждый пиксель занимает 2 байта
    //(С учетом неотображаемых пикселей)
    unsigned int frame_byte_size = _frame_len*0x2;

    //Инициализация цепочки
    DMA_Configure_chain(
        &chain[0],
        &chain[1],
        0, 0, 1,
        (unsigned int * )_frame_addr,
        frame_byte_size/2 );

    DMA_Configure_chain(
        &chain[1],
        &chain[0],
        1, 1, 1,
        (unsigned int * )(_frame_addr + ( frame_byte_size/2 )),
        frame_byte_size/2 );

    //Загрузка первого звена в регистры DMA VPOUT
    DMA_Load_chain(&chain[0]);

}


//Запуск/приостановка порта VPOUT и запуск канала DMA
void VPOUT_Start(unsigned int _enable){

    CSR_VPOUT_CH  = _enable ? (CSR_VPOUT_CH|(1)) : CSR_VPOUT_CH ;
    CSR_VPOUT  = _enable  ? (CSR_VPOUT|(1<<30)) : (CSR_VPOUT&(~(1<<30)));

}
