﻿//                                     VPOUT
//     Данный проект демонстрирует настройку порта VPOUT
//     и последующий вывод изображения на дисплей MC-LCD-TOUCH
//     Проект применим к отладочному модулю NVCOM-02TEM-3U

#include "memory_nvcom02_t.h"
#include "VPOUT.h"

//Адрес изображения формата BMP
#define PICT_ADDR 0x80200000

//Максимально возможный размер кадра для AT070TN92 - 1200*650 = 780000*0x2
//[байт] = 0x177000
//Так как, передаваемый кадр хранится по адресу 0x80000000,
//то загружаемый файл BMP хранится по адресу > 0x80000000 + 0x00177000

void Get_pict(unsigned int _frame_addr) {
  //Проверка на соответствие файла формату BMP не осуществляется
  //Необходимые параметры загружаемого файла
  //Разрешение WIDTH*HEIGHT - 800*480
  //Глубина цвета 16 [бит/пиксель]
  //Без сжатия

  //Смещение изображения от начала файла находится
  //в заголовке файла - по адресу (начало_файла + 0xA)
  //Порядок следования пикселей в формате BMP :  СЛЕВА НАПРАВО И СНИЗУ ВВЕРХ

  //Смещение изображения от начала файла
  unsigned int bmp_header_offset = *(unsigned short *)(PICT_ADDR + 0xA);
  //Указатель на начало картинки + смещение
  unsigned short *pict_data = (unsigned short *)(PICT_ADDR + bmp_header_offset);
  //Смещение указателя чтения из загруженной картинки на начало предпоследней
  //строки
  pict_data += ((HEIGHT - 1) * WIDTH);
  //Указатель на начало первого действительного пикселя
  unsigned short *ptr_x = (unsigned short *)(_frame_addr) + FRAME_OFFSET;

  unsigned int x, y;
  //Кадр
  for (y = 0; y < HEIGHT; y++) { // 480
    for (x = 0; x < WIDTH; x++)  // 800
      *ptr_x++ = *pict_data++;

    //Перемещение на начало предыдущей строки
    pict_data -= 2 * WIDTH;
    //Отступ при записи следующей строки пикселей
    ptr_x += (HS_BLANKING - H_START + H_AFTERSYNC);
  }
}

void Frame_BW_Grid(unsigned int _frame_addr) {

  unsigned int x, y;
  unsigned short *ptr_x = (unsigned short *)(_frame_addr) + FRAME_OFFSET;
  unsigned short pix = WHITE;

  //Кадр
  for (y = 0; y < HEIGHT; y++) { // 480
    if (!(y % 40))
      pix = ~pix;
    for (x = 0; x < WIDTH; x++) { // 800
      if (!(x % 40))
        pix = ~pix;

      *ptr_x++ = pix;
    }
    ptr_x += (HS_BLANKING - H_START + H_AFTERSYNC);
  }
}

// Задать значение регистру Status сопроцессора CP0
void SetCP0_Status(unsigned int value) {
  asm volatile("mtc0 %0, $12" ::"r"(value));
}

int main(void) {

  // Настройка прерываний для VPOUT и DMA_VPOUT
  SetCP0_Status((1 << 10) | (1)); // Разрешаем прерывания вообще - CP0.Status[0]
  // = 1 и прерывания от VPOUT и DMA VPOUT -
  // CP0.Status[10] = 1 IM[7:2]= 10 -
  // прерывания, объединенные по ИЛИ в
  // псевдорегистре QSTR0;

  MASKR0 |= (1 << 19) |
            (1 << 18); // Разрешаем прерывания от DMA VPOUT - MASKR0[19] = 1

  // Очистка RG Causes
  asm volatile("mtc0 $0,$13");

  // Вектора прерывания размещаются во внутренней памяти CRAM (адреса типа
  // 0xB8000000);
    CSR |= (1<<1);

    //Генерация кадра
    //Frame_BW_Grid(FRAME_ADDR);
    Get_pict(FRAME_ADDR);

    //Инициализация VPOUT
    VPOUT_Init(
        DIV_VCLKO_OUT,//Fc   Freq_VCLKO_out = 240*1e6/_div_VCLKO_out
        H_START,      //Hstart - количество тактов VCLKO_out(пиксельной синхронизации), при которых HSYNC_out = 0
        H_END,        //Период сигнала HSYNC_out в размерности пиксельной синхронизации; Th = HEND*Tc (где Tc - период VCLKO_out)
        V_START,      //VSTART - количество тактов HSYNC_out(строчной синхронизации), при которых VSYNC_out = 0
        V_END,        //Период сигнала VSYNC_out в размерности строчной синхронизации; Tv = VEND*Th (где Th - период HSYNC_out)
        FRAME_ADDR ,  //Адрес памяти, который хранит видеокадр
        FRAME_LEN     //Длина кадра в пикселях
    );

    //Запуск сначала цепочки DMA, а затем пуск VPOUT
    VPOUT_Start(1);

    while(1);

    return 0;

}

